﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class Counter : MVRScript
    {
		private JSONStorableFloat myCount;
		private EventTrigger myEventTrigger;
		private int myTriggered = 0;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>Counter</b></size>\nLogicBrick that triggers once after <i>Trigger</i> was called a number of times. Resets automatically when count is reached. Call <i>Reset</i> to reset manually.</color>",
				200.0f, true
			);

			myTriggered = 0;

			myCount = Utils.SetupSliderInt(this, "Count", 3, 1, 50, false);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Reset", Reset, false);
			Utils.SetupAction(this, "Reset", Reset);
			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;

			if (myEventTrigger != null)
				myEventTrigger.Remove();
		}

		private void Trigger()
		{
			if (myTriggered < int.MaxValue)
				++myTriggered;
			if (myTriggered >= myCount.val)
			{
				myEventTrigger.Trigger();
				myTriggered = 0;
			}
		}

		private void Reset()
		{
			myTriggered = 0;
		}
    }
}
